// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use std::fs::File;
use std::io::{BufReader, BufWriter};

use png::{Encoder, Decoder, ColorType, BitDepth};

pub struct Image {
	pub width: u32,
	pub height: u32,
	pub data: Box<[u8]>,
}

impl Image {
	pub fn new(width: u32, height: u32, channels: u32) -> Image {
		Image {
			width, height,
			data: vec![0; (width * height * channels) as usize].into_boxed_slice(),
		}
	}

	// Did some copy-pasting from a previous project which was copy-pasted from the documentation, each time some modifications made
	pub fn load(path: &str, format: ColorType) -> Image {
		let decoder = Decoder::new(BufReader::new(File::open(path).unwrap()));

		let mut reader = decoder.read_info().unwrap();
		let mut data = vec![0; reader.output_buffer_size()];
		let info = reader.next_frame(&mut data).unwrap();
		data.truncate(info.buffer_size());

		assert!(info.bit_depth == BitDepth::Eight);
		assert!(info.color_type == format);

		Image {
			width: info.width,
			height: info.height,
			data: data.into_boxed_slice(),
		}
	}

	pub fn save(&self, path: &str, colour_type: ColorType) {
		let mut encoder = Encoder::new(BufWriter::new(File::create(path).unwrap()), self.width, self.height);
		encoder.set_color(colour_type);
		encoder.set_depth(BitDepth::Eight);
		encoder.write_header().unwrap().write_image_data(&self.data).unwrap();
	}
}
