// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use std::f32::consts::TAU;

use glam::{Vec2, Vec4};

use super::super::Particle;
use crate::powerup::RADIUS;

pub struct Reload {
	pos: Vec2,
	size: f32,
	green: f32,
	r: f32,
	theta: f32,
	vel: f32,
	rot_vel: f32,
	time: f32,
	max_time: f32,
}

impl Reload {
	pub fn new(pos: Vec2) -> Reload {
		let max_time = rand::random::<f32>() + 2.0;
		Reload {
			pos,
			size: (rand::random::<f32>() + 1.0) * 0.03125,
			green: rand::random::<f32>() * 0.125 + 0.3125,
			r: rand::random::<f32>() * RADIUS,
			theta: rand::random::<f32>() * TAU,
			vel: rand::random::<f32>() * 0.25 + 0.625,
			rot_vel: rand::random::<f32>() * 2.0 + 1.0,
			time: max_time,
			max_time,
		}
	}
}

impl Particle for Reload {
	fn get_pos(&self) -> Vec2 {
		let (sin, cos) = self.theta.sin_cos();
		self.pos + self.r * Vec2::new(cos, sin)
	}

	fn get_size(&self) -> f32 { self.size }
	fn get_colour(&self) -> Vec4 { Vec4::new(0.5, self.green, 0.125, self.time / self.max_time) }

	fn update(&mut self, dt: f32) -> bool {
		self.vel *= (1.0 - dt).max(0.0);
		self.r += self.vel * dt;
		self.theta -= self.rot_vel * dt;
		self.time -= dt;
		self.time >= 0.0
	}
}
